%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% This code uses the RK4 method to solve an SIR disease model
%% with a periodic contact rate.
%% Written by Calistus N. Ngonghala
%% West Virginia University, USA
%% email: cngongha@mix.wvu.edu; ngonghala@yahoo.com
%% July 1, 2010
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function SIRPeriodic
clear all;
close all;
clc
% Request parameter values and initial data from user.
prompt={'Enter the value of \beta_0 ',...
        'Enter the value of \beta_1 ',...
        'Enter the value of \mu ',...
        'Enter the value of \nu ',...
        'Enter the initial time, t_1 ',...
        'Enter the final time, t_f ',...
        'Enter the number of time steps, Nsteps ',...
        'Enter the initial proportion of susceptibles individuals, S_1: ',...
        'Enter the initial proportion of infectious individuals, I_1: ',...
        'Enter the initial proportion of Recovered individuals, R_1: '};
name='Parameters and Initial values for an SIR Model with a Periodic Contact Rate';
numlines=1;
defaultanswer={'1800','0.03','0.02','100','0','200','20000','0.0556','0.001','0'};
options.Resize='on';
options.WindowStyle='normal';
options.Interpreter='tex';
parameters=inputdlg(prompt,name,numlines,defaultanswer,options);
%%%%%%%%%%%%%%%%%%%%%% Convert cell arrays to numbers %%%%%%%%%%%%%%%%%%%%%
beta_0 = str2num(parameters{1});
beta_1 = str2num(parameters{2});
mu     = str2num(parameters{3});
nu     = str2num(parameters{4});
t_1    = str2num(parameters{5});
t_f    = str2num(parameters{6});
Nsteps = str2num(parameters{7});
S_1    = str2num(parameters{8});
I_1    = str2num(parameters{9});
R_1    = str2num(parameters{10});

x_0 = [S_1 I_1 R_1 t_1]; % Vector of initial values.
m = size(x_0,1);
if m == 1
   x_0 = x_0';
end
h = (t_f - t_1)/Nsteps;  % Calculate the step size
t(1) = t_1;
x(:,1) = x_0;     % Initialize solution vector

% Solve system using RK4
for i = 1:1:Nsteps
   k_1 = h*SIR(t(i), x(:,i)); % Compute K_1
   k_2 = h*SIR(t(i)+h/2, x(:,i)+0.5*k_1); % Compute K_2
   k_3 = h*SIR(t(i)+h/2, x(:,i)+0.5*k_2); % Compute K_3
   k_4 = h*SIR(t(i)+h, x(:,i)+k_3);       % Compute K_4
   x(:,i+1) = x(:,i) + (k_1 + 2*k_2 + 2*k_3 + k_4)/6;
   t(i+1) = t_1 + i*h;
end

if beta_1 == 0.0
    R_0 = beta_0/(nu + mu) % Compute the basic reproduction number.
    figure(1) % This figure plots the negative natural log of S against time
    plot(t,-log(x(1,:)),'g','LineWidth',2);
    xlabel('Time')
    ylabel('-log(S)');
    figure(2) % Plot negative natural log of I against time.
    plot(t,-log(x(2,:)),'b','LineWidth',2);
    xlabel('Time')
    ylabel('-log(I)')
    figure(3) % Plot negative natural log of I against negative natural log of S.
    plot(-log(x(1,:)),-log(x(2,:)),'LineWidth',2);
    xlabel('-log(S)')
    ylabel('-log(I)')
elseif beta_1 > 0.089
    S = x(1,round(max(size(x))*0.5):max(size(x))-1);
    I = x(2,round(max(size(x))*0.5):max(size(x))-1);
    T = t(round(max(size(x))*0.5):max(size(x))-1);
    figure(1)
    plot(T,-log(I),'LineWidth',2);
    xlabel('Time')
    ylabel('-log(I)')
    figure(2)
    plot(-log(S),-log(I),'LineWidth',2);
    xlabel('-log(S)')
    ylabel('-log(I)')
else

    S = x(1,round(max(size(x))*0.9):max(size(x))-1);
    I = x(2,round(max(size(x))*0.9):max(size(x))-1);
    T = t(round(max(size(x))*0.9):max(size(x))-1);

    figure(1)
    plot(T,-log(I),'LineWidth',2);
    xlabel('Time');
    ylabel('-log(I)')

    figure(2)
    plot(-log(S),-log(I),'LineWidth',2);
    xlabel('-log(S)');
    ylabel('-log(I)')
end

    function dx = SIR(t,x)
        S = x(1);
        I = x(2);
        R = x(3);
        TT = x(4);
        dS = mu - beta(TT)*S*I - mu*S;
        dI = beta(TT)*S*I - (mu + nu)*I;
        dR = nu*I - mu*R;
        dTT = 1;
        dx = [dS;dI;dR;dTT];
        
        function b = beta(t)
            b = beta_0*(1 + beta_1*cos(2*pi*t));
        end
    end
end
